@@ -57,14 +57,20 @@ static void randomdelay(void)
  * -Frank.
  */
 
+static size_t scanned;
+static size_t readnbd;
+
+static void flush_cmd(void)
+{
+    scanned = readnbd = (size_t) 0U;
+}
+
 int sfgets(void)
 {
     struct pollfd pfd;
     int pollret;
     ssize_t readnb;
     signed char seen_r = 0;
-    static size_t scanned;
-    static size_t readnbd;
     
     if (scanned > (size_t) 0U) {       /* support pipelining */
         readnbd -= scanned;        
@@ -72,7 +78,11 @@ int sfgets(void)
         scanned = (size_t) 0U;
     }
     pfd.fd = clientfd;
+#ifdef __APPLE_CC__
     pfd.events = POLLIN | POLLERR | POLLHUP;
+#else
+    pfd.events = POLLIN | POLLPRI | POLLERR | POLLHUP;
+#endif
     while (scanned < cmdsize) {
         if (scanned >= readnbd) {      /* nothing left in the buffer */
             pfd.revents = 0;
@@ -85,7 +95,7 @@ int sfgets(void)
                 (pfd.revents & (POLLERR | POLLHUP | POLLNVAL)) != 0) {
                 return -2;
             }
-            if ((pfd.revents & POLLIN) == 0) {
+            if ((pfd.revents & (POLLIN | POLLPRI)) == 0) {
                 continue;
             }
             if (readnbd >= cmdsize) {
@@ -358,6 +368,7 @@ void parser(void)
             addreply_noformat(234, "AUTH TLS OK.");
             doreply();
             if (tls_cnx == NULL) {
+                flush_cmd();
                 (void) tls_init_new_session();
             }
             goto wayout;
@@ -473,7 +484,12 @@ void parser(void)
                     addreply_noformat(501, MSG_STAT_FAILURE);
                 } else {
                     const off_t size = (off_t) strtoull(arg, NULL, 10);
-                    doallo(size);
+                    
+                    if (size < (off_t) 0) {
+                        addreply_noformat(501, MSG_STAT_FAILURE);                        
+                    } else {
+                        doallo(size);
+                    }
                 }
 #endif
             } else if (!strcmp(cmd, "pwd") || !strcmp(cmd, "xpwd")) {
