@@ -48,7 +48,7 @@ GLOBAL0(signed char candownload);                /* if non-zero, don't let the u
 GLOBAL0(double load);                    /* for reporting what the load was */
 GLOBAL(time_t noopidle, (time_t) - 1);    /* when we started to receive NOOP */
 GLOBAL(unsigned int firstport, 1024U);        /* first and last ports to use, if we're */
-GLOBAL(unsigned int lastport, 65534U);            /* packet filter friendly. */
+GLOBAL(unsigned int lastport, 65535U);
 GLOBAL(signed char dot_write_ok, 1);           /* -x option */
 GLOBAL(signed char dot_read_ok, 1);            /* -X option */
 GLOBAL0(signed char dot_read_anon_ok);            /* -z option */
@@ -171,13 +171,13 @@ GLOBAL0(char *atomic_prefix);
 #endif
 
 #ifdef WITH_RFC2640
-GLOBAL(char utf8, 0);	/* 0: ascii 1: utf-8 */
+GLOBAL(signed char utf8, 0);	/* 0: ascii 1: utf-8 */
 GLOBAL(char *charset_fs, NULL);
 GLOBAL(char *charset_client, NULL);
-GLOBAL(iconv_t iconv_fd_fs2client, NULL);
-GLOBAL(iconv_t iconv_fd_fs2utf8, NULL);
-GLOBAL(iconv_t iconv_fd_client2fs, NULL);
-GLOBAL(iconv_t iconv_fd_utf82fs, NULL);
+GLOBAL(iconv_t iconv_fd_fs2client, (iconv_t) -1);
+GLOBAL(iconv_t iconv_fd_fs2utf8, (iconv_t) -1);
+GLOBAL(iconv_t iconv_fd_client2fs, (iconv_t) -1);
+GLOBAL(iconv_t iconv_fd_utf82fs, (iconv_t) -1);
 #endif
 
 #ifndef WITH_TLS
@@ -188,4 +188,7 @@ GLOBAL0(void * tls_data_cnx);
 #ifdef NON_ROOT_FTP
 GLOBAL0(const char *home_directory);
 #endif
-    
+
+#ifndef MINIMAL
+GLOBAL0(unsigned long cwd_failures);
+#endif
