@@ -50,7 +50,7 @@ static char *pw_mysql_escape_string(MYSQL * const id_sql_server,
     }
     from_len = strlen(from);
     to_len = from_len * 2U + (size_t) 1U;
-    if ((to = malloc(to_len + 2U)) == NULL) {
+    if ((to = malloc(to_len + (size_t) 2U)) == NULL) {
         return NULL;
     }
     t = zrand();
@@ -70,7 +70,7 @@ static char *pw_mysql_escape_string(MYSQL * const id_sql_server,
 #ifdef HAVE_MYSQL_REAL_ESCAPE_STRING
     tolen = mysql_real_escape_string(id_sql_server, to, from, from_len);
 #else
-    /* MySQL 3.22.x and earlier are obsolete. Better use 3.23.x or 4.x */
+    /* MySQL 3 is obsolete. */
     tolen = mysql_escape_string(to, from, from_len);    
 #endif
     if (tolen >= to_len || 
@@ -195,13 +195,24 @@ static int pw_mysql_connect(MYSQL ** const id_sql_server)
     if ((*id_sql_server = mysql_init(NULL)) == NULL) {
         down:
         if (server_down == 0) {
+            const char *mysql_err = "mysql_init()";
+            
+            if (*id_sql_server != NULL) {
+                mysql_err = mysql_error(*id_sql_server);
+            }
             server_down++;
-            logfile(LOG_ERR, MSG_SQL_DOWN);
+            logfile(LOG_ERR, MSG_SQL_DOWN " [%s]", mysql_err);
         }
         return -1;
     }
     if (mysql_real_connect(*id_sql_server, server, user, pw,
-                           db, port, socket_path, 0) == NULL) {
+                           db, port, socket_path,
+#ifdef CLIENT_MULTI_STATEMENTS
+                           CLIENT_MULTI_STATEMENTS
+#else
+                           0
+#endif
+                           ) == NULL) {
         goto down;
     }
     if (mysql_ping(*id_sql_server) != 0) {
@@ -273,8 +284,13 @@ static char *pw_mysql_getquery(MYSQL * const id_sql_server,
     bye:
     if (qresult != NULL) {
         mysql_free_result(qresult);
-    }
-    
+#ifdef CLIENT_MULTI_STATEMENTS
+        while (mysql_next_result(id_sql_server) == 0) {
+            qresult = mysql_store_result(id_sql_server);
+            mysql_free_result(qresult);
+        }
+#endif
+    }    
     return answer;    
 }
 
@@ -410,28 +426,40 @@ void pw_mysql_check(AuthResult * const result,
         crypto_plain++;
     }
     if (crypto_crypt != 0) {
-        register const char *crypted;
+        const char *crypted;
 
         if ((crypted = (const char *) crypt(password, spwd)) != NULL &&
             strcmp(crypted, spwd) == 0) {
             goto auth_ok;
         }
     }
-#if MYSQL_VERSION_ID < 40100
     if (crypto_mysql != 0) {
+#if MYSQL_VERSION_ID < 40100 || defined(USE_OLD_MYSQL_SCRAMBLING)
         unsigned long hash_res[2];
         char scrambled_password[MYSQL_CRYPT_LEN];
-        
+     
+# if MYSQL_VERSION_ID < 40100
         hash_password(hash_res, password);
+# else
+        hash_password(hash_res, password, strlen(password));        
+# endif     
         snprintf(scrambled_password, sizeof scrambled_password, "%08lx%08lx", 
                  hash_res[0], hash_res[1]);
+#else
+        char scrambled_password[42]; /* 2 * 20 (sha1 hash size) + 2 */
+
+# if MYSQL_VERSION_ID >= 40100 && MYSQL_VERSION_ID < 40101
+        make_scrambled_password(scrambled_password, password, 1, NULL);
+# else
+        make_scrambled_password(scrambled_password, password);
+# endif
+#endif
         if (strcmp(scrambled_password, spwd) == 0) {
             goto auth_ok;
         }
     }
-#endif
     if (crypto_md5 != 0) {
-        register const char *crypted;
+        const char *crypted;
 
         if ((crypted = (const char *) crypto_hash_md5(password, 1)) != NULL &&
             strcmp(crypted, spwd) == 0) {
@@ -544,7 +572,7 @@ void pw_mysql_check(AuthResult * const result,
     }
 #endif
     result->slow_tilde_expansion = !tildexp;
-    result->auth_ok =- result->auth_ok;
+    result->auth_ok = -result->auth_ok;
     bye:
     if (committed == 0) {
         (void) pw_mysql_simplequery(id_sql_server, MYSQL_TRANSACTION_END);
@@ -553,8 +581,12 @@ void pw_mysql_check(AuthResult * const result,
         mysql_close(id_sql_server);
     }
     free((void *) spwd);
-    free((void *) uid);
-    free((void *) gid); 
+    if (uid != sql_default_uid) {
+        free((void *) uid);
+    }
+    if (gid != sql_default_gid) {
+        free((void *) gid); 
+    }
     free((void *) dir);
 #ifdef QUOTAS
     free((void *) sqta_fs);
