@@ -169,7 +169,7 @@ static void help(void)
          "                [-q <upload ratio>] [-Q <download ratio>]\n"
          "                [-r <allow client ip>/<mask>] [-R <deny client ip>/<mask>]\n"
          "                [-i <allow local ip>/<mask>] [-I <deny local ip>/<mask>]\n"
-	 "                [-y <max number of concurrent sessions>]\n"
+     "                [-y <max number of concurrent sessions>]\n"
          "                [-z <hhmm>-<hhmm>] [-m]\n"
          "\n"
          "pure-pw usermod <login> -f <passwd file> -u <uid> [-g <gid>]\n"
@@ -179,7 +179,7 @@ static void help(void)
          "                [-q <upload ratio>] [-Q <download ratio>]\n"
          "                [-r <allow client ip>/<mask>] [-R <deny client ip>/<mask>]\n"
          "                [-i <allow local ip>/<mask>] [-I <deny local ip>/<mask>]\n"
-	 "                [-y <max number of concurrent sessions>]\n"	 
+     "                [-y <max number of concurrent sessions>]\n"     
          "                [-z <hhmm>-<hhmm>] [-m]\n"
          "\n"
          "pure-pw userdel <login> [-f <passwd file>] [-m]\n"
@@ -194,7 +194,7 @@ static void help(void)
          "\n"
          "-d <home directory> : chroot user (recommended)\n"
          "-D <home directory> : don't chroot user\n"
-	 "-<option> '' : set this option to unlimited\n"
+     "-<option> '' : set this option to unlimited\n"
          "-m : also update the " DEFAULT_PW_DB " database\n"
          "For a 1:10 ratio, use -q 1 -Q 10\n"
          "To allow access only between 9 am and 6 pm, use -z 0900-1800\n"
@@ -217,7 +217,7 @@ static void pw_zrand_probe(void)
     static const char * const devices[] = {
         "/dev/arandom", "/dev/urandom", "/dev/random", NULL
     };
-    register const char * const *device = devices;
+    const char * const *device = devices;
     
     do {
         if (access(*device, F_OK | R_OK) == 0) {
@@ -246,8 +246,10 @@ static unsigned int pw_zrand(void)
 #endif        
         ) {
         nax:
-#ifdef HAVE_RANDOM
-        return (unsigned int) random();        
+#ifdef HAVE_ARC4RANDOM
+        return (unsigned int) arc4random();
+#elif defined HAVE_RANDOM
+        return (unsigned int) random();
 #else
         return (unsigned int) rand();
 #endif
@@ -265,7 +267,7 @@ static char *best_crypt(const char * const pwd)
 {
     static const char crcars[64] =
         "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789./";
-    register const char *crypted;
+    const char *crypted;
     
     if ((crypted = (const char *)      /* Blowfish */
          crypt("test", "$2a$07$1234567890123456789012")) != NULL &&        
@@ -432,7 +434,7 @@ static int parse_pw_line(char *line, PWInfo * const pwinfo)
         return 0;
     }
     if (*line != 0) {
-	pwinfo->per_user_max = (unsigned int) strtoul(line, NULL, 10);
+    pwinfo->per_user_max = (unsigned int) strtoul(line, NULL, 10);
         if (pwinfo->per_user_max > 0U) {
             pwinfo->has_per_user_max = 1;
         }
@@ -639,9 +641,9 @@ static int add_new_pw_line(FILE * const fp2, const PWInfo * const pwinfo)
         return -1;
     }
     if (pwinfo->has_per_user_max != 0) {
-	if (fprintf(fp2, "%u", pwinfo->per_user_max) < 0) {
-	    return -1;
-	}
+    if (fprintf(fp2, "%u", pwinfo->per_user_max) < 0) {
+        return -1;
+    }
     }
     if (fprintf(fp2, PW_LINE_SEP) < 0) {    
         return -1;
@@ -1113,7 +1115,7 @@ static int do_show(const char * const file, const PWInfo * const pwinfo)
            "Allowed client IPs : %s\n"
            "Denied  client IPs : %s\n"           
            "Time restrictions  : %04u-%04u (%s)\n"
-	   "Max sim sessions   : %u (%s)\n"
+           "Max sim sessions   : %u (%s)\n"
            "\n",
            fetched_info.login,
            fetched_info.pwd,
@@ -1140,7 +1142,7 @@ static int do_show(const char * const file, const PWInfo * const pwinfo)
            SHOW_IFEN(fetched_info.has_time, fetched_info.time_begin),
            SHOW_IFEN(fetched_info.has_time, fetched_info.time_end),
            SHOW_STATE(fetched_info.has_time),
-	   SHOW_IFEN(fetched_info.has_per_user_max, fetched_info.per_user_max),
+           SHOW_IFEN(fetched_info.has_per_user_max, fetched_info.per_user_max),
            SHOW_STATE(fetched_info.per_user_max));
     
     return 0;
@@ -1161,9 +1163,7 @@ static int do_passwd(const char * const file,
         fprintf(stderr, "Error with entering password - aborting\n");        
         return PW_ERROR_ENTER_PASSWD_PW_ERROR;
     }    
-    do_usermod(file, pwinfo);
-    
-    return 0;
+    return do_usermod(file, pwinfo);
 }
 
 static int do_mkdb(const char *dbfile, const char * const file)
@@ -1251,11 +1251,10 @@ static void init_zrand(void)
     gettimeofday(&tv, &tz);
 #ifdef HAVE_SRANDOMDEV
     srandomdev();
-#endif    
-#ifdef HAVE_RANDOM
-    srandom((unsigned int) (tv.tv_sec ^ tv.tv_usec));
+#elif defined(HAVE_RANDOM)
+    srandom((unsigned int) (tv.tv_sec ^ tv.tv_usec ^ (getpid() << 16)));
 #else
-    srand((unsigned int) (tv.tv_sec ^ tv.tv_usec));
+    srand((unsigned int) (tv.tv_sec ^ tv.tv_usec ^ (getpid() << 16)));
 #endif
 }
 
@@ -1324,8 +1323,9 @@ int main(int argc, char *argv[])
         pwinfo.login = NULL;
     }
     filter_pw_line_sep(pwinfo.login);
-    while ((fodder = getopt(argc, argv, 
-                            "c:d:D:f:g:hi:I:mn:N:q:Q:r:R:t:T:u:y:z:")) != -1) {
+    while ((fodder =
+            getopt(argc, argv, 
+                   "c:d:D:f:F:g:hi:I:mn:N:q:Q:r:R:t:T:u:y:z:")) != -1) {
         switch(fodder) {
         case 'c' : {
             if ((pwinfo.gecos = strdup(optarg)) == NULL) {
@@ -1370,6 +1370,12 @@ int main(int argc, char *argv[])
             }
             break;
         }
+        case 'F' : {
+            if ((dbfile = strdup(optarg)) == NULL) {
+                no_mem();
+            }
+            break;
+        }
         case 'g' : {
             struct group *gr;
             
@@ -1500,14 +1506,14 @@ int main(int argc, char *argv[])
             }
             break;
         }
-	case 'y' : {
-	    if ((pwinfo.per_user_max = (unsigned int) strtoul(optarg, NULL, 10)) <= 0U) {
+    case 'y' : {
+        if ((pwinfo.per_user_max = (unsigned int) strtoul(optarg, NULL, 10)) <= 0U) {
                 pwinfo.has_per_user_max = -1;
             } else {
                 pwinfo.has_per_user_max = 1;
             }
-	    break;
-	}
+        break;
+    }
         case 'z' : {
             if (sscanf(optarg, "%u-%u", 
                        &pwinfo.time_begin, &pwinfo.time_end) == 2 &&
@@ -1554,9 +1560,9 @@ int main(int argc, char *argv[])
             ret |= do_mkdb(dbfile, file);
         }        
     } else if (strcasecmp(action, "passwd") == 0) {
-        do_passwd(file, &pwinfo);
+        ret = do_passwd(file, &pwinfo);
         if (with_mkdb != 0) {
-            do_mkdb(dbfile, file);
+            ret |= do_mkdb(dbfile, file);
         }        
     } else if (strcasecmp(action, "show") == 0) {
         ret = do_show(file, &pwinfo);
